//
//  ViewController.swift
//  Books
//
//  Created by Bart Jacobs on 21/01/2021.
//

import UIKit
import CoreData

final class BooksViewController: UIViewController {

    // MARK: - Properties
    
    @IBOutlet private var booksLabel: UILabel!
    
    // MARK: -
    
    private lazy var persistentContainer: NSPersistentContainer = {
        NSPersistentContainer(name: "Books")
    }()

    deinit {
        // Remove Observer
        NotificationCenter.default.removeObserver(self)
    }
    
    // MARK: - View Life Cycle
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        persistentContainer.loadPersistentStores { [weak self] persistentStoreDescription, error in
            if let error = error {
                print("Unable to Add Persistent Store")
                print("\(error), \(error.localizedDescription)")
                
            } else {
                self?.fetchBooks()
            }
        }
        
        // Observe Managed Object Context
        NotificationCenter.default.addObserver(forName: .NSManagedObjectContextDidSave,
                                               object: persistentContainer.viewContext,
                                               queue: .main) { [weak self] _ in
            self?.fetchBooks()
        }
    }
    
    // MARK: - Navigation
    
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if let viewController = segue.destination as? AddBookViewController {
            viewController.managedObjectContext = persistentContainer.viewContext
        }
    }
    
    // MARK: - Helper Methods

    private func fetchBooks() {
        // Create Fetch Request
        let fetchRequest: NSFetchRequest<Book> = Book.fetchRequest()
        
        // Perform Fetch Request
        persistentContainer.viewContext.perform {
            do {
                // Execute Fetch Request
                let result = try fetchRequest.execute()
                
                // Update Books Label
                self.booksLabel.text = "\(result.count) Books"
                
            } catch {
                print("Unable to Execute Fetch Request, \(error)")
            }
        }
    }

}
